/*
 * Use consistent with the GNU GPL is permitted,
 * provided that this copyright notice is
 * preserved in its entirety in all copies and derived works.
 *
 * History:
 *
 * 2004-03-01   Eddi De Pieri      Adapted for h4000 using h3900_lcd.c 
 * 2004         Shawn Anderson     Lcd hacking on h4000
 * see h3900_lcd.c for more history.
 *
 */

#include <linux/types.h>
#include <asm/arch/hardware.h>  /* for pxa-regs.h (__REG) */
#include <linux/platform_device.h>
#include <asm/arch/pxa-regs.h>  /* LCCR[0,1,2,3]* */
#include <asm/arch/bitfield.h>  /* for pxa-regs.h (Fld, etc) */
#include <asm/arch/pxafb.h>     /* pxafb_mach_info, set_pxa_fb_info */
#include <asm/mach-types.h>     /* machine_is_h4000 */
#include <linux/lcd.h>          /* lcd_device */
#include <linux/backlight.h>    /* backlight_device */
#include <linux/err.h>
#include <linux/delay.h>

#include <asm/arch/h4000-gpio.h>
#include <asm/arch/h4000-asic.h>
#include <asm/hardware/ipaq-asic3.h>
#include <linux/soc/asic3_base.h>

extern struct platform_device h4000_asic3;
static int saved_brightness;

static int h4000_lcd_get_power(struct lcd_device *lm)
{
	/* Get the LCD panel power status (0: full on, 1..3: controller
	 * power on, flat panel power off, 4: full off) */

	if (ipaq_asic3_read_gpio_status_b(&h4000_asic3.dev) & GPIOB_LCD_PCI) {
		if (ipaq_asic3_read_gpio_out_b(&h4000_asic3.dev) & GPIOB_LCD_ON)
			return 0;
		else
			return 2;
	} else
		return 4;
}

static int h4000_lcd_set_power(struct lcd_device *lm, int power)
{
	 /* Enable or disable power to the LCD (0: on; 4: off) */

	if ( power < 1 ) {
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev,
				GPIOC_LCD_3V3_ON, GPIOC_LCD_3V3_ON);
		mdelay(30);
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev, 
				GPIOC_LCD_N3V_EN, GPIOC_LCD_N3V_EN);
		mdelay(5);
		ipaq_asic3_set_gpio_out_b(&h4000_asic3.dev,
				GPIOB_LCD_ON, GPIOB_LCD_ON);
		mdelay(50);
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev,
				GPIOC_LCD_5V_EN, GPIOC_LCD_5V_EN);
		mdelay(5);
	} else {
		mdelay(5);
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev, GPIOC_LCD_5V_EN, 0);
		mdelay(50);
		ipaq_asic3_set_gpio_out_b(&h4000_asic3.dev, GPIOB_LCD_ON, 0);
		mdelay(5);
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev, GPIOC_LCD_N3V_EN,0);
		mdelay(100);
		ipaq_asic3_set_gpio_out_c(&h4000_asic3.dev, GPIOC_LCD_3V3_ON,0);
	}

	if ( power < 4 ) {
		mdelay(17);     // Wait one from before turning on
		ipaq_asic3_set_gpio_out_b(&h4000_asic3.dev,
				GPIOB_LCD_PCI, GPIOB_LCD_PCI);
	} else {
		ipaq_asic3_set_gpio_out_b (&h4000_asic3.dev, GPIOB_LCD_PCI, 0);
		mdelay(30);     // Wait before turning off
	}
	
	return 0;
}

static int h4000_lcd_get_contrast(struct lcd_device *ld)
{
	/* Get the current contrast setting (0-max_contrast) */
//DBG*/	printk("%s: not implemented yet\n", __FUNCTION__);
	return 0;
}

static int h4000_lcd_set_contrast(struct lcd_device *ld, int contrast)
{
	/* Set LCD panel contrast */
//DBG*/	printk("%s: not implemented yet\n", __FUNCTION__);
	return 0;
}

static int h4000_bl_get_power(struct backlight_device *bl)
{
	/* Get the power status (0: on, 1..3: power saving modes; 4: off) */
	// GPIO_NR_H4000_LCD_PWM, CKEN0_PWM0, GPIOB_BACKLIGHT_POWER_ON
	if ((ipaq_asic3_read_gpio_status_b(&h4000_asic3.dev) &
				GPIOB_BACKLIGHT_POWER_ON))
		return 0;
	else
		return 4;
}

static int h4000_bl_set_power(struct backlight_device *bl, int power)
{
	/* Enable or disable power to the LCD (0: on; 4: off) */

	if (power < 1)
		ipaq_asic3_set_gpio_out_b(&h4000_asic3.dev,
			GPIOB_BACKLIGHT_POWER_ON, GPIOB_BACKLIGHT_POWER_ON);
	else
		ipaq_asic3_set_gpio_out_b(&h4000_asic3.dev,
			GPIOB_BACKLIGHT_POWER_ON, 0);
	return 0;
}

static int h4000_bl_get_brightness(struct backlight_device *bd)
{
	/* Get current backlight brightness */
	u32 x, y;

	x = PWM_PWDUTY0 & 0x3ff;
	y = PWM_PERVAL0 & 0x3ff;
	// To avoid division, we'll approximate y to nearest 2^n-1.
	// PocketPC is using 0xfe as PERVAL0 and we're using 0x3ff.
	x <<= (10 - fls (y));
	return x;
}

static int h4000_bl_set_brightness(struct backlight_device *bd, int brightness)
{
	/* Set backlight brightness (0..max_brightness) */
	if (brightness > 0x3ff)
		brightness = 0x3ff;

	/* LCD brightness is driven by PWM0.
	 * We'll set the pre-scaler to 8, and the period to 1024, this
	 * means the backlight refresh rate will be 3686400/(8*1024) =
	 * 450 Hz which is quite enough.
	 */
	PWM_CTRL0 = 7;            /* pre-scaler */
	PWM_PWDUTY0 = brightness; /* duty cycle */
	PWM_PERVAL0 = 0x3ff;      /* period */
	if (brightness)
		pxa_set_cken(CKEN0_PWM0, 1);
	else
		pxa_set_cken(CKEN0_PWM0, 0);
	return 0;
}

static struct lcd_properties h4000_lcd_properties =
{
	.owner          = THIS_MODULE,
	.get_power      = h4000_lcd_get_power,
	.set_power      = h4000_lcd_set_power,
	.max_contrast   = 7,
	.get_contrast   = h4000_lcd_get_contrast,
	.set_contrast   = h4000_lcd_set_contrast,
};

static struct backlight_properties h4000_bl_properties =
{
	.owner          = THIS_MODULE,
	.get_power      = h4000_bl_get_power,
	.set_power      = h4000_bl_set_power,
	.max_brightness = 0x3ff,
	.get_brightness = h4000_bl_get_brightness,
	.set_brightness = h4000_bl_set_brightness,
};

static struct lcd_device *h4000_lcd_dev;
static struct backlight_device *h4000_bl_dev;

static int h4000_lcd_probe(struct device * dev)
{
	h4000_lcd_dev = lcd_device_register("pxa2xx-fb", NULL,
			&h4000_lcd_properties);
	h4000_bl_dev = backlight_device_register("pxa2xx-fb", NULL,
			&h4000_bl_properties);
	if (IS_ERR(h4000_lcd_dev) || IS_ERR(h4000_bl_dev)) {
		printk("h4000_lcd_probe: error registering devices\n");
		return -1;
	}

	return 0;
}

static int h4000_lcd_remove(struct device * dev)
{
	h4000_lcd_set_power(h4000_lcd_dev, 4);
	h4000_bl_set_power(h4000_bl_dev, 4);
	backlight_device_unregister(h4000_bl_dev);
	lcd_device_unregister(h4000_lcd_dev);

	return 0;
}

static int h4000_lcd_suspend(struct device * dev, pm_message_t state)
{
	saved_brightness = h4000_bl_get_brightness(h4000_bl_dev);
	h4000_bl_set_power(h4000_bl_dev, 4);
	h4000_lcd_set_power(h4000_lcd_dev, 4);
	return 0;
}

static int h4000_lcd_resume(struct device * dev)
{
	h4000_lcd_set_power(h4000_lcd_dev, 0);
	h4000_bl_set_power(h4000_bl_dev, 0);
	h4000_bl_set_brightness(h4000_bl_dev, saved_brightness);
	return 0;
}

static struct device_driver h4000_lcd_driver = {
	.name     = "h4000_lcd",
	.bus      = &platform_bus_type,
	.probe    = h4000_lcd_probe,
	.remove   = h4000_lcd_remove,
	.suspend  = h4000_lcd_suspend,
	.resume   = h4000_lcd_resume,
};

static int h4000_lcd_init(void)
{
	if (!machine_is_h4000())
		return -ENODEV;

	return driver_register(&h4000_lcd_driver);
}

static void h4000_lcd_exit(void)
{
	lcd_device_unregister(h4000_lcd_dev);
	backlight_device_unregister(h4000_bl_dev);
	driver_unregister(&h4000_lcd_driver);
}

module_init(h4000_lcd_init);
module_exit(h4000_lcd_exit);

MODULE_AUTHOR("h4000 port team h4100-port@handhelds.org");
MODULE_DESCRIPTION("Framebuffer driver for iPAQ H4000");
MODULE_LICENSE("GPL");

/* vim: set ts=8 tw=80 shiftwidth=8 noet: */

