/*
 * Hardware definitions for HP iPAQ Handheld Computers
 *
 * Copyright 2000-2003 Hewlett-Packard Company.
 *
 * Use consistent with the GNU GPL is permitted,
 * provided that this copyright notice is
 * preserved in its entirety in all copies and derived works.
 *
 * COMPAQ COMPUTER CORPORATION MAKES NO WARRANTIES, EXPRESSED OR IMPLIED,
 * AS TO THE USEFULNESS OR CORRECTNESS OF THIS CODE OR ITS
 * FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * Author: Jamey Hicks.
 *
 * History:
 *
 * 2003-05-14	Joshua Wise        Adapted for the HP iPAQ H1900
 * 2002-08-23   Jamey Hicks        Adapted for use with PXA250-based iPAQs
 * 2001-10-??   Andrew Christian   Added support for iPAQ H3800
 *                                 and abstracted EGPIO interface.
 *
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/tty.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/lcd.h>
#include <linux/backlight.h>
#include <linux/fb.h>
#include <../drivers/video/pxafb.h>

#include <asm/hardware.h>
#include <asm/setup.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/arch/ipaq.h>
#include <asm/arch/himalaya_asic.h>
#include <asm/arch/pxafb.h>

#include "himalaya_lcd.h"
//#include "../../common/ipaq/asic2_base.h"
#include <linux/soc/asic3_base.h>

//extern struct platform_device h3900_asic2;
extern struct platform_device h3900_asic3;

#define HIMALAYA_ATI(b) (*((unsigned int *) (b + HIMALAYA_ATI_CHIP_BASE)))
#define HIMALAYA_VIDMEM(b) (*((unsigned char *) (b + HIMALAYA_ATI_VIDMEM_BASE )))

static void himalaya_init_ATI(void)
{
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CLK_PIN_CNTL ) = 0x401BF;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_PLL_REF_FB_DIV ) = 0x50500D04;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_PLL_CNTL ) = 0x4B000200;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_SCLK_CNTL) = 0xB11;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_PCLK_CNTL) = 0xB1;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CLK_TEST_CNTL ) = 0x1;

	HIMALAYA_ATI( HIMALAYA_ATI_REG_PWRMGT_CNTL ) = 0xFFFF11C5;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_MC_FB_LOCATION ) = 0x15FF1000;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_LCD_FORMAT )  = 3;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GRAPHIC_CTRL ) = 0x01D41C06;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GRAPHIC_OFFSET ) = 0x100000;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GRAPHIC_PITCH ) = 0x1E0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_TOTAL ) = 0x14F010F;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_ACTIVE_H_DISP ) = 0xF00000;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_ACTIVE_V_DISP ) = 0x14F000F;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GRAPHIC_H_DISP ) = 0xF00000;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GRAPHIC_V_DISP ) = 0x14F000F;

	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_SS) = 0x80150014;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_LS) = 0xA0FB00F7;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_REV) = 0xA;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_DCLK) = 0xA1700030;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_GS) = 0xC0080007;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_VPOS_GS) = 0x80007;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_GCLK) = 0x8015010F;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_GOE) = 0x0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_FRAME) = 0x0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_FRAME_VPOS) = 0x0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_LCDD_CNTL1) = 0x0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_LCDD_CNTL2) = 0x3FFFF;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GENLCD_CNTL1) = 0xFFF003;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GENLCD_CNTL2) = 0x3;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_DISP_DEBUG) = 0x100;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_DEFAULT_COUNT) = 0x0;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_LCD_BACKGROUND_COLOR) = 0xFF00;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_CRTC_PS1_ACTIVE) = 0x41060010;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_GENLCD_CNTL3) = 0x102AA;
	HIMALAYA_ATI( HIMALAYA_ATI_REG_DISP_DEBUG) = 0x800000;
}

static int himalaya_lcd_set_power( struct lcd_device *ld, int setp )
{
	if ( setp ) {
		printk("himalaya LCD power on\n");
		/*BL	 print_ATI_type*/

		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_5V_ON, 0);
		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_ON, 0);
		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_NV_ON, 0);
		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_9V_ON, 0);

		msleep(500);

		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_ON, GPIOB_LCD_ON);
		msleep(17);

		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_NV_ON, GPIOB_LCD_NV_ON);
		himalaya_init_ATI();
		msleep(30);

		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_9V_ON, GPIOB_LCD_9V_ON);
		msleep(30);

		ipaq_asic3_set_gpio_out_b(&h3900_asic3.dev, GPIOB_LCD_5V_ON, GPIOB_LCD_5V_ON);
		msleep(30);

		/*
		HIMALAYA_VIDMEM( 0x0) = 0xff;
		HIMALAYA_VIDMEM( 0x1) = 0xff;
		HIMALAYA_VIDMEM( 0x2) = 0x00;
		HIMALAYA_VIDMEM( 0x3) = 0x00;
	*/
	}
	return 0;
}

static int himalaya_lcd_get_power( struct lcd_device *ld, int *psetp )
{
	printk("himalaya_lcd_get_power\n");
	if (psetp) {
			*psetp = 0;
		/*
		if (ipaq_asic3_read_gpio_b (&h3900_asic3.dev) & GPIOB_LCD_ON)
			*psetp = 1;
		else
			*psetp = 0;
			*/
	}
	return 0;
}

static int himalaya_lcd_set_enable( struct lcd_device *lm, int setp )
{
	printk("himalaya_lcd_set_enable\n");
	/*
	if ( setp ) {
		mdelay(17);     // Wait one from before turning on
		ipaq_asic3_set_gpio_out_b (&h3900_asic3.dev, GPIOB_LCD_ON, GPIOB_LCD_ON);
	} else {
		ipaq_asic3_set_gpio_out_b (&h3900_asic3.dev, GPIOB_LCD_ON, 0);
		mdelay(30);     // Wait before turning off
	}
	*/
	return 0;
}

static int himalaya_lcd_get_enable( struct lcd_device *lm, int *psetp )
{
	printk("himalaya_lcd_get_enable\n");
	if (psetp) {
			*psetp = 0;
		/*
		if (ipaq_asic3_read_gpio_b (&h3900_asic3.dev) & GPIOB_LCD_ON)
			*psetp = 1;
		else
			*psetp = 0;
			*/
	}
	return 0;
}

static int himalaya_backlight_set_power (struct backlight_device *bd, int on)
{
	if (on) {
		ipaq_asic3_set_gpio_out_b (&h3900_asic3.dev, GPIOB_FL_PWR_ON, GPIOB_FL_PWR_ON);
	} else {
		ipaq_asic3_set_gpio_out_b (&h3900_asic3.dev, GPIOB_FL_PWR_ON, 0);
	}
	return 0;
}

static int himalaya_backlight_get_power (struct backlight_device *bd)
{
	if (ipaq_asic3_read_gpio_out_b (&h3900_asic3.dev) & GPIOB_FL_PWR_ON)
		return 1;
	else
		return 0;
}

/* read LCCR0: 0x00000081 */
/* read LCCR1: 0x0b10093f -- BLW=0x0b, ELW=0x10, HSW=2, PPL=0x13f */
/* read LCCR2: 0x051008ef -- BFW=0x05, EFW=0x10, VSW=2, LPP=0xef */
/* read LCCR3: 0x0430000a */
static struct pxafb_mach_info himalayafb_info = {
	.pixclock =	221039,	/* clock period in ps */
	.bpp =		16,
	.xres =		240,
	.yres =		320,
	.hsync_len =	3,
	.vsync_len =	3,
	.left_margin =	12,
	.upper_margin =	6,
	.right_margin =	17,
	.lower_margin =	17,
	.sync =		0, /* both horiz and vert active low sync */
	.lccr0 =	(LCCR0_PAS | LCCR0_LDM | LCCR0_SFM | LCCR0_IUM | LCCR0_EFM | LCCR0_QDM | LCCR0_BM  | LCCR0_OUM),
	.lccr3 =	(LCCR3_16BPP)
};

static struct lcd_device *himalaya_lcd_device;

static struct lcd_properties himalaya_lcd_props = {
	.set_power     = himalaya_lcd_set_power,
	.get_power     = himalaya_lcd_get_power,
#if 0
	.set_contrast    = himalaya_lcd_set_enable,
	.get_contrast    = himalaya_lcd_get_enable,
#endif
};

static struct backlight_device *himalaya_backlight_device;

static struct backlight_properties himalaya_backlight_props = {
	.set_power     = himalaya_backlight_set_power,
	.get_power     = himalaya_backlight_get_power,
#if 0
	.get_brightness = himalaya_backlight_get_brightness,
	.set_brightness = himalaya_backlight_set_brightness,
#endif
};

int __init
himalaya_lcd_init (void)
{
	if (! machine_is_himalaya() )
		return -ENODEV;

	printk("LCD register!\n");
	set_pxa_fb_info (&himalayafb_info);
	himalaya_lcd_device = lcd_device_register("himalaya-fb", NULL,
		&himalaya_lcd_props);
	himalaya_backlight_device = backlight_device_register("himalaya-fb", NULL,
		&himalaya_backlight_props);

	return 0;
}

void __exit
himalaya_lcd_exit (void)
{
	lcd_device_unregister (&himalaya_lcd_device);
	backlight_device_unregister (&himalaya_backlight_device);
}

module_init (himalaya_lcd_init);
module_exit (himalaya_lcd_exit);
