/*
 * 
 * Hardware definitions for the HTC Magician
 *
 * Copyright 2006 Philipp Zabel
 *
 * Based on hx4700.c
 * Copyright 2005 SDG Systems, LLC
 *
 * Use consistent with the GNU GPL is permitted,
 * provided that this copyright notice is
 * preserved in its entirety in all copies and derived works.
 *
 */


#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/delay.h>

#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <asm/arch/htcmagician-gpio.h>
#include <asm/arch/pxa-regs.h>
#include <asm/hardware/gpio_keys.h>
#include <asm/arch/udc.h>
#include <asm/arch/audio.h>
#include <asm/arch/irda.h>
#include <asm/arch/ohci.h>

#include <asm/hardware/ipaq-asic3.h>
#include <linux/soc/asic3_base.h>

#include "../generic.h"
//#include "htcmagician_phone.h"

/*
 * Bluetooth - Relies on other loadable modules, like ASIC3 and Core,
 * so make the calls indirectly through pointers. Requires that the
 * htcmagician bluetooth module be loaded before any attempt to use
 * bluetooth (obviously).
 */

/*
 * IRDA
 */

static void htcmagician_irda_transceiver_mode(struct device *dev, int mode)
{
	unsigned long flags;

	local_irq_save(flags);
#if 0
	if (mode & IR_OFF)
		SET_HTCMAGICIAN_GPIO(IR_ON, 1); /* is this correct? */
	else
		SET_HTCMAGICIAN_GPIO(IR_ON, 0);
#endif
	local_irq_restore(flags);
}

static struct pxaficp_platform_data htcmagician_ficp_platform_data = {
	.transceiver_cap  = IR_SIRMODE | IR_OFF,
	.transceiver_mode = htcmagician_irda_transceiver_mode,
};

/* Initialization code */

static void __init htcmagician_map_io(void)
{

	pxa_map_io();

	//ffuart_device.dev.platform_data = &htcmagician_pxa_phone_funcs;
}

static void __init htcmagician_init_irq(void)
{
	/* int irq; */

	pxa_init_irq();
}

/* Core Hardware Functions */

#if 0

static struct resource pxa_cs2_resources[] = {
	[0] = {
		.start = PXA_CS2_PHYS,		/* */
		.end = PXA_CS2_PHYS + 0x1000000, 	/* */
		.flags = IORESOURCE_MEM,
	},
};

#endif

struct platform_device htcmagician_core = {
	.name		= "htcmagician-core",
	.id		= 0,
//	.num_resources	= ARRAY_SIZE(pxa_cs2_resources),
//	.resource	= pxa_cs2_resources,
	.dev = {
		.platform_data = NULL,
	},
};

/* USB Device Controller */

static int
udc_detect(void)
{
	int enabled = GET_HTCMAGICIAN_GPIO(USBC_DETECT) ? 1 : 0;
	printk (KERN_NOTICE "udc_detect: %d\n", enabled);
	return enabled;
}

static void
udc_enable(int cmd) 
{
	switch (cmd)
	{
		case PXA2XX_UDC_CMD_DISCONNECT:
			printk (KERN_NOTICE "USB cmd disconnect\n");
//			SET_HTCMAGICIAN_GPIO(USB_PUEN, 0);
			break;

		case PXA2XX_UDC_CMD_CONNECT:
			printk (KERN_NOTICE "USB cmd connect\n");
//			SET_HTCMAGICIAN_GPIO(USB_PUEN, 1);
			break;
	}
}

static struct pxa2xx_udc_mach_info htcmagician_udc_mach_info = {
	.udc_is_connected = udc_detect,
	.udc_command      = udc_enable,
};

/* GPIO Keys */

static struct gpio_keys_button htcmagician_button_table[] = {
	{ KEY_POWER, GPIO_NR_HTCMAGICIAN_KEY_ON, 1 },
	{ KEY_F8 /* CANCEL */, GPIO_NR_HTCMAGICIAN_KEY_PHONE_HANGUP, 0 },
	{ KEY_F5 /* KEY_CONTACTS */, GPIO_NR_HTCMAGICIAN_KEY_CONTACTS, 0 },
	{ KEY_F9 /* KEY_CALENDAR */, GPIO_NR_HTCMAGICIAN_KEY_CALENDAR, 0 },
	{ KEY_F10 /* KEY_CAMERA */, GPIO_NR_HTCMAGICIAN_KEY_CAMERA, 0 },
	{ KEY_UP, GPIO_NR_HTCMAGICIAN_KEY_UP, 0 },
	{ KEY_DOWN, GPIO_NR_HTCMAGICIAN_KEY_DOWN, 0 },
	{ KEY_LEFT, GPIO_NR_HTCMAGICIAN_KEY_LEFT, 0 },
	{ KEY_RIGHT, GPIO_NR_HTCMAGICIAN_KEY_RIGHT, 0 },
	{ KEY_KPENTER /* KEY_SELECT */, GPIO_NR_HTCMAGICIAN_KEY_ENTER, 0 },
	{ KEY_F9 /* KEY_RECORD */, GPIO_NR_HTCMAGICIAN_KEY_RECORD, 0 },
	{ KEY_VOLUMEUP, GPIO_NR_HTCMAGICIAN_KEY_VOLUMEUP, 0 },
	{ KEY_VOLUMEDOWN, GPIO_NR_HTCMAGICIAN_KEY_VOLUMEDOWN, 0 },
	{ KEY_F7 /* KEY_PHONE */, GPIO_NR_HTCMAGICIAN_KEY_PHONE_LIFT, 0 },
};

static struct gpio_keys_platform_data htcmagician_gpio_keys_data = {
	.buttons = htcmagician_button_table,
	.nbuttons = ARRAY_SIZE(htcmagician_button_table),
};

static struct platform_device htcmagician_gpio_keys = {
	.name = "gpio-keys",
	.dev = {
		.platform_data = &htcmagician_gpio_keys_data,
	},
};

/* LCD */

static struct platform_device htcmagician_lcd = {
	.name = "htcmagician-lcd",
	.id = -1,
	.dev = {
		.platform_data = NULL,
	},
};

/* Touchscreen, maybe change to SoC later */

static struct platform_device htcmagician_ts = {
	.name = "htcmagician-ts",
	.dev = {
		.platform_data = NULL,
	},
};

#if 0

/* Phone */

static struct platform_device htcmagician_phone = {
	.name = "htcmagician-phone",
	.id = -1,
	.dev = {
		.platform_data = &phone_funcs,
	},
};

static struct platform_device htcmagician_power = {
	.name = "htcmagician-power",
	.id = -1,
};

static struct platform_device htcmagician_flash = {
	.name = "htcmagician-flash",
	.id = -1,
};

#endif

/* USB OHCI */

static int htcmagician_ohci_init(struct device *dev)
{
	/* missing GPIO setup here */

	/* no idea what this does, got the values from haret */
	UHCHR = (UHCHR | UHCHR_SSEP2 | UHCHR_PCPL | UHCHR_CGR) &
		~(UHCHR_SSEP1 | UHCHR_SSEP3 | UHCHR_SSE);

	return 0;
}

static struct pxaohci_platform_data htcmagician_ohci_platform_data = {
	.port_mode	= PMM_PERPORT_MODE,
	.init		= htcmagician_ohci_init,
};

static struct platform_device *devices[] __initdata = {
	&htcmagician_core,
//	&htcmagician_power,
	&htcmagician_gpio_keys,
//	&htcmagician_lcd,
	&htcmagician_ts,
//	&htcmagician_flash,
//	&htcmagician_phone,
};


static void __init htcmagician_init(void)
{
	/* hx4700 has GPSR, GPCR, GPDR, GAFR and MSC as set by bootloader here */

  /* TODO */

	platform_add_devices(devices, ARRAY_SIZE(devices));
        pxa_set_ohci_info(&htcmagician_ohci_platform_data);
	pxa_set_udc_info(&htcmagician_udc_mach_info);
	pxa_set_ficp_info(&htcmagician_ficp_platform_data);
}


MACHINE_START(MAGICIAN, "HTC Magician")
        /* Maintainer xanadux.org */
	.phys_ram	= 0xa0000000,
	.phys_io	= 0x40000000,
	.io_pg_offst	= (io_p2v(0x40000000) >> 18) & 0xfffc,
        .boot_params	= 0xa0000100,
        .map_io		= htcmagician_map_io,
        .init_irq	= htcmagician_init_irq,
        .timer = &pxa_timer,
        .init_machine = htcmagician_init,
MACHINE_END
