/*
 * LCD driver for HTC Magician
 * 
 * Authors: Giuseppe Zompatori <giuseppe_zompatori@yahoo.it>
 * 
 * based on previous work, see below:
 * 
 * Machine initialization for Dell Axim X3
 *
 * Authors: Andrew Zabolotny <zap@homelink.ru>
 *
 * For now this is mostly a placeholder file; since I don't own an Axim X3
 * it is supposed the project to be overtaken by somebody else. The code
 * in here is *supposed* to work so that you can at least boot to the command
 * line, but there is no guarantee.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/notifier.h>
#include <linux/lcd.h>
#include <linux/backlight.h>
#include <linux/err.h>
#include <linux/delay.h>
#include <linux/fb.h>

#include <asm/arch/htcmagician-gpio.h>
#include <asm/mach-types.h>
#include "asm/arch/pxa-regs.h"
#include "asm/arch/pxafb.h"

#include "htcmagician_lcd.h"

#include <asm/arch/hardware.h> // for lcdclk_freq

static void
htcmagician_lcd_hw_on(void)
{
	printk("htcmagician_lcd_hw_on bl=%d\n", htcmagician_backlight_on);

}

static void
htcmagician_lcd_hw_off(void)
{
	printk("htcmagician_lcd_hw_off\n");

}

static int htcmagician_lcd_set_power (struct lcd_device *lm, int setp)
{
	printk("htcmagician_lcd_set_power called.\n");

	printk("FB_BLANK_UNBLANK %d FB_BLANK_NORMAL %d\n", FB_BLANK_UNBLANK, FB_BLANK_NORMAL);
	printk("FB_BLANK_VSYNC_SUSPEND %d FB_BLANK_HSYNC_SUSPEND %d\n", FB_BLANK_VSYNC_SUSPEND, FB_BLANK_HSYNC_SUSPEND);
	printk("FB_BLANK_POWERDOWN %d\n", FB_BLANK_POWERDOWN);
	printk("htcmagician LCD power %d\n",setp);
	switch (setp) {
	case FB_BLANK_UNBLANK:
	case FB_BLANK_NORMAL:
		htcmagician_lcd_hw_on();
		break;
	case FB_BLANK_VSYNC_SUSPEND:
	case FB_BLANK_HSYNC_SUSPEND:
		break;
	case FB_BLANK_POWERDOWN:
		htcmagician_lcd_hw_off();
		break;
	}
	htcmagician_lcd_power=setp;

	return 0;
}

static int htcmagician_lcd_get_power (struct lcd_device *lm)
{
	printk("htcmagician_lcd_get_power called.\n");
	return htcmagician_lcd_power;
}

static struct pxafb_mach_info htcmagician_fb_info1 =
{
//	.pixclock =     230769,  // TODO: check that LCCR4=0x80000000 ( 104Mhz/(k=2)/10kHz*pixclock/2/100*1e6-1=pcd=0x5.)
	.pixclock =     96153,  // TODO: check that LCCR4=0x80000000 ( 104Mhz/(k=2)/10kHz*pixclock/2/100*1e6-1=pcd=0x5.)
	.bpp =          16,  // 
	.xres =         240, // PPL + 1 0x1df
	.yres =         320, // LPP + 1 0x27f
	.hsync_len =    4,  // HSW + 1 0x3
	.vsync_len =    1,   // VSW + 1 0x0
	.left_margin =  20,  // BLW + 1 0x13
	.upper_margin = 7,   // BFW     0x07
	.right_margin = 8,  // ELW + 1 0x07
	.lower_margin = 8,   // EFW 0x08
	.sync =         0,   // ???
	.lccr0 = 0x00000080, // yes that's ugly, but it's due a limitation of the
//	.lccr3 = 0x04700005 // pxafb driver assuming pxa25x fbs...
	.lccr3 = 0x00400000  // bpp and pcd removed. TODO: use the values defined in pxafb.h
};

static struct pxafb_mach_info htcmagician_fb_info2 =
{
	.pixclock =     96153,  // TODO: check that LCCR4=0x80000000 ( 104Mhz/(k=2)/10kHz*pixclock/2/100*1e6-1=pcd=0x5.)
//PCD=0x4
	.bpp =          16,  // 
	.xres =         240, // PPL + 1 0x1df
	.yres =         320, // LPP + 1 0x27f
	.hsync_len =    11,  // HSW + 1 0x3
	.vsync_len =    3,   // VSW + 1 0x0
	.left_margin =  19,  // BLW + 1 0x13
	.upper_margin = 2,   // BFW     0x07
	.right_margin = 10,  // ELW + 1 0x07
	.lower_margin = 2,   // EFW 0x08
	.sync =         0,   // ???
// wince: 00000081 (LCCR0_Pas = 0x80, LCCR0_ENB = 0x1)
	.lccr0 = 0x00000080, // enb removed, see pxafb.h
// wince: 0x04300004 (BPP=4, HSP,VSP, PCD=4)
	.lccr3 = 0x00000000  // bpp, hsp, vsp and pcd removed
};

/*
 * pH5's magician needs info2. awelux' magician needs info1
 * one is supposed to have a Samsung LTP280QV, the other a
 * Toppoly TD028STEB1 lcd.
 */
#define htcmagician_fb_info htcmagician_fb_info2

struct lcd_properties htcmagician_lcd_properties =
{
	.owner = THIS_MODULE,
	.set_power = htcmagician_lcd_set_power,
	.get_power = htcmagician_lcd_get_power,
	/* @@ more here @@ */
};

#define LCD_SET_BRIGHT(br)	\
	PWM_CTRL0 = 1;		\
	PWM_PERVAL0 = 0xc8;	\
	PWM_PWDUTY0 = br

static int htcmagician_backlight_set_brightness(struct backlight_device *bd, int bright)
{
	if (bright >= 0 && bright <= 0xc7)
	{
	 printk("setting brightness=%d of %d\n",bright,0xc8);
	 LCD_SET_BRIGHT( bright );
	}
	else 
	 printk("brightness=%d is too high.\n",bright);

	return 0;
}

static int htcmagician_backlight_get_brightness(struct backlight_device *bd )
{
	int bright = (int) PWM_PWDUTY0;

	return bright;
}


static int htcmagician_backlight_set_power (struct backlight_device *bm, int on)
{
	printk("htcmagician_backlight_set_power called.\n");

	htcmagician_backlight_power=on;

	printk("htcmagician_backlight_set_power(%d)\n", on);
	switch (on) {
	case FB_BLANK_UNBLANK:
	case FB_BLANK_NORMAL:
		htcmagician_backlight_on=1;
		break;
	case FB_BLANK_VSYNC_SUSPEND:
	case FB_BLANK_HSYNC_SUSPEND:
		break;
	case FB_BLANK_POWERDOWN:
		htcmagician_backlight_on=0;
		break;
	}

	return 0;
}

static int htcmagician_backlight_get_power (struct backlight_device *bm)
{
	printk("htcmagician_backlight_get_power called.\n");
	return htcmagician_backlight_power;
}

static struct backlight_properties htcmagician_backlight_properties =
{
	.owner          = THIS_MODULE,
	.set_power      = htcmagician_backlight_set_power,
	.get_power      = htcmagician_backlight_get_power,
	.get_brightness = htcmagician_backlight_get_brightness,
	.set_brightness = htcmagician_backlight_set_brightness,
	.max_brightness = 0xc8,
};

static struct lcd_device *pxafb_lcd_device;
static struct backlight_device *pxafb_backlight_device;

static int __init
htcmagician_lcd_init (void)
{
	set_pxa_fb_info(&htcmagician_fb_info);

	pxafb_lcd_device = lcd_device_register("pxafb", NULL, &htcmagician_lcd_properties);
	pxafb_backlight_device = backlight_device_register("pxafb", NULL,&htcmagician_backlight_properties);

//	if (IS_ERR (pxafb_backlight_device)) {
//		lcd_device_unregister (pxafb_lcd_device);
//		return PTR_ERR (pxafb_backlight_device);
//	}

	printk("htcmagician_lcd_init: htcmagician_backlight_on %d\n", htcmagician_backlight_on);

	return 0;
}

#if 0
static void
htcmagician_lcd_suspend()
{
	htcmagician_lcd_hw_off();
}

static void
htcmagician_lcd_resume()
{
	htcmagician_lcd_hw_on();
}

struct w100_tg_info htcmagician_tg_info = {
	.suspend	= htcmagician_lcd_suspend,
	.resume		= htcmagician_lcd_resume,
};
#endif

static void __exit
htcmagician_lcd_exit (void)
{
	lcd_device_unregister (pxafb_lcd_device);
	backlight_device_unregister (pxafb_backlight_device);
}

module_init (htcmagician_lcd_init);
module_exit (htcmagician_lcd_exit);

MODULE_AUTHOR("xanadux.org");
MODULE_DESCRIPTION("HTC Magician Core framebuffer driver");
MODULE_LICENSE("GPL");

