/*
 * Hardware definitions for HTC Universal
 *
 * Copyright 2000-2003 Hewlett-Packard Company.
 *
 * Use consistent with the GNU GPL is permitted,
 * provided that this copyright notice is
 * preserved in its entirety in all copies and derived works.
 *
 * COMPAQ COMPUTER CORPORATION MAKES NO WARRANTIES, EXPRESSED OR IMPLIED,
 * AS TO THE USEFULNESS OR CORRECTNESS OF THIS CODE OR ITS
 * FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * History:
 * 2004-??-??  Shawn Anderson    Derived the from aximx3.c aximx5.c e7xx.c 
 *                               h1900.c h2200.c h3900.c h5400.c and friends.
 * 2004-04-01  Eddi De Pieri     Move lcd stuff so it can be used as a module.
 * 2004-04-19  Eddi De Pieri     Moving to new 2.6 standard 
 *                               (platform_device / device_driver structure)
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>

#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/setup.h>

#include <asm/mach/irq.h>
#include <asm/mach/arch.h>

#include <asm/arch/serial.h>
#include <asm/arch/bitfield.h>
#include <asm/arch/pxa-regs.h>
#include <asm/arch/ipaq.h>
#include <asm/arch/pxafb.h>
#include <asm/arch/irda.h>
#include <linux/soc/asic3_base.h>
#include <asm/arch/htcuniversal-gpio.h>
#include <asm/arch/htcuniversal-init.h>
#include <asm/arch/htcuniversal-asic.h>

#include <asm/hardware/gpio_keys.h>
#include <asm/hardware/ipaq-asic3.h>

#include "../generic.h"

#include "htcuniversal_bt.h"
#include "htcuniversal_phone.h"

/*
 * IRDA
 */

static void htcuniversal_irda_transceiver_mode(struct device *dev, int mode)
{
 /* */
}

static struct pxaficp_platform_data htcuniversal_ficp_platform_data = {
	.transceiver_cap  = IR_SIRMODE | IR_FIRMODE,
	.transceiver_mode = htcuniversal_irda_transceiver_mode,
};

/*
 * Bluetooth - Relies on other loadable modules, like ASIC3 and Core,
 * so make the calls indirectly through pointers. Requires that the
 * hx4700 bluetooth module be loaded before any attempt to use
 * bluetooth (obviously).
 */

static struct htcuniversal_bt_funcs bt_funcs;

static void
htcuniversal_bt_configure( int state )
{
	if (bt_funcs.configure != NULL)
		bt_funcs.configure( state );
}

static struct htcuniversal_phone_funcs phone_funcs;

static void
htcuniversal_phone_configure( int state )
{
	if (phone_funcs.configure != NULL)
		phone_funcs.configure( state );
}

//void htcuniversal_ll_pm_init(void);

static struct platform_device htcuniversal_lcd       = { .name = "htcuniversal_lcd", };
static struct platform_device htcuniversal_kbd       = { .name = "htcuniversal_kbd", };
//static struct platform_device htcuniversal_buttons   = { .name = "htcuniversal_buttons", };
static struct platform_device htcuniversal_ts        = { .name = "htcuniversal_ts", };
//static struct platform_device htcuniversal_bt        = { .name = "htcuniversal_bt", };
//static struct platform_device htcuniversal_phone        = { .name = "htcuniversal_phone", };
static struct platform_device htcuniversal_power        = { .name = "htcuniversal_power", };
static struct platform_device htcuniversal_udc       = { .name = "htcuniversal_udc", };

/* Bluetooth */

static struct platform_device htcuniversal_bt = {
	.name = "htcuniversal_bt",
	.id = -1,
	.dev = {
		.platform_data = &bt_funcs,
	},
};

static struct platform_device htcuniversal_phone = {
	.name = "htcuniversal_phone",
	.id = -1,
	.dev = {
		.platform_data = &phone_funcs,
	},
};

/* PXA2xx Keys */

static struct gpio_keys_button htcuniversal_button_table[] = {
	{ KEY_POWER, GPIO_NR_HTCUNIVERSAL_KEY_ON_N, 1 },
};

static struct gpio_keys_platform_data htcuniversal_pxa_keys_data = {
	.buttons = htcuniversal_button_table,
	.nbuttons = ARRAY_SIZE(htcuniversal_button_table),
};

static struct platform_device htcuniversal_pxa_keys = {
	.name = "gpio-keys",
	.dev = {
		.platform_data = &htcuniversal_pxa_keys_data,
	},
};

/* Core Hardware Functions */

struct platform_device htcuniversal_core = {
	.name		= "htcuniversal_core",
	.id		= 0,
	.dev = {
		.platform_data = NULL,
	},
};

static struct platform_device *devices[] __initdata = {
	&htcuniversal_core,
//	&htcuniversal_flash,
	&htcuniversal_pxa_keys,
};

static struct platform_device *htcuniversal_asic3_devices[] __initdata = {
	&htcuniversal_lcd,
//	&htcuniversal_buttons,
	&htcuniversal_ts,
	&htcuniversal_bt,
	&htcuniversal_kbd,
	&htcuniversal_phone,
	&htcuniversal_power,
	&htcuniversal_udc,
};

static struct asic3_platform_data htcuniversal_asic3_platform_data = {

   /* Setting ASIC3 GPIO registers to the below initialization states
    * HTC Universal asic3 information: 
    * http://wiki.xda-developers.com/index.php?pagename=UniversalASIC3
    * http://wiki.xda-developers.com/index.php?pagename=ASIC3
    *
    * dir:	Direction of the GPIO pin. 0: input, 1: output.
    *      	If unknown, set as output to avoid power consuming floating input nodes
    * init:	Initial state of the GPIO bits
    *
    * These registers are configured as they are on Wince.
    */
        .gpio_a = {
		.dir            = 0x865f, // 
		.init           = 0x8001, // TODO: 0x0000
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x0000, //
		.sleep_conf     = 0x000c,
        },
        .gpio_b = {
		.dir            = 0xc142, //
		.init           = 0x8842, // TODO: 0x0900
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x0000, //
                .sleep_conf     = 0x000c,
        },
        .gpio_c = {
                .dir            = 0xc7e7, //
                .init           = 0xc6e0, // TODO: 0x8000
                .sleep_out      = 0x0000,
                .batt_fault_out = 0x0000,
		.alt_function   = 0x0007, // GPIOC_LED_RED | GPIOC_LED_GREEN | GPIOC_LED_BLUE
                .sleep_conf     = 0x000c,
        },
        .gpio_d = {
		.dir            = 0xffc0, //
		.init           = 0x7840, // TODO: 0x0000
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x0000, //
		.sleep_conf     = 0x0008,
        },
	.bus_shift = 1,

	.child_platform_devs     = htcuniversal_asic3_devices,
	.num_child_platform_devs = ARRAY_SIZE(htcuniversal_asic3_devices),
};

static struct resource htcuniversal_asic3_resources[] = {
	[0] = {
		.start	= HTCUNIVERSAL_ASIC3_GPIO_PHYS,
		.end	= HTCUNIVERSAL_ASIC3_GPIO_PHYS + IPAQ_ASIC3_MAP_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= HTCUNIVERSAL_IRQ(ASIC3_EXT_INT),
		.end	= HTCUNIVERSAL_IRQ(ASIC3_EXT_INT),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start  = HTCUNIVERSAL_ASIC3_MMC_PHYS,
		.end    = HTCUNIVERSAL_ASIC3_MMC_PHYS + IPAQ_ASIC3_MAP_SIZE,
		.flags  = IORESOURCE_MEM,
	},
	[3] = {
		.start  = HTCUNIVERSAL_IRQ(ASIC3_SDIO_INT_N),
		.flags  = IORESOURCE_IRQ,
	},
};

struct platform_device htcuniversal_asic3 = {
	.name           = "asic3",
	.id             = 0,
	.num_resources  = ARRAY_SIZE(htcuniversal_asic3_resources),
	.resource       = htcuniversal_asic3_resources,
	.dev = { .platform_data = &htcuniversal_asic3_platform_data, },
};
EXPORT_SYMBOL(htcuniversal_asic3);



static struct pxafb_mach_info sony_acx526akm __initdata= {
	.pixclock		= 96153,
	.xres			= 480,
	.yres			= 640,
	.bpp			= 16,
	.hsync_len		= 4,
	.vsync_len		= 1,
	.left_margin		= 20,
	.right_margin		= 8,
	.upper_margin		= 7,
	.lower_margin		= 8,

//	.sync			= FB_SYNC_HOR_LOW_ACT|FB_SYNC_VERT_LOW_ACT,

	/* fixme: this is a hack, use constants instead. */
	.lccr0			= 0x00000080,
	.lccr3			= 0x00400000,
//	.lccr4			= 0x80000000,
};

static void __init htcuniversal_init_irq(void)
{
	pxa_init_irq();
}

static struct platform_pxa_serial_funcs htcuniversal_pxa_bt_funcs = {
	.configure = htcuniversal_bt_configure,
};
static struct platform_pxa_serial_funcs htcuniversal_pxa_phone_funcs = {
	.configure = htcuniversal_phone_configure,
};

static void __init htcuniversal_map_io(void)
{
	pxa_map_io();

	btuart_device.dev.platform_data = &htcuniversal_pxa_bt_funcs;
	ffuart_device.dev.platform_data = &htcuniversal_pxa_phone_funcs;
#if 0
	/* Wake up enable. */
	PWER = PWER_GPIO0 | PWER_RTC;
	/* Wake up on falling edge. */
	PFER = PWER_GPIO0 | PWER_RTC;
	/* Wake up on rising edge. */
	PRER = 0;
	/* 3.6864 MHz oscillator power-down enable */
	PCFR = PCFR_OPDE;
#endif
}

static void __init htcuniversal_init(void)
{
	set_pxa_fb_info(&sony_acx526akm);

	pxa_gpio_mode( GPIO_NR_HTCUNIVERSAL_I2C_SCL_MD );
	pxa_gpio_mode( GPIO_NR_HTCUNIVERSAL_I2C_SDA_MD );

#ifdef CONFIG_PM
//	htcuniversal_ll_pm_init();
#endif

	platform_device_register(&htcuniversal_asic3);
	platform_add_devices( devices, ARRAY_SIZE(devices) );
	pxa_set_ficp_info(&htcuniversal_ficp_platform_data);
}

MACHINE_START(HTCUNIVERSAL, "HTC Universal")
	/* Maintainer xanadux.org */
	.phys_ram	= 0xa0000000,
	.phys_io	= 0x40000000,
	.io_pg_offst	= (io_p2v(0x40000000) >> 18) & 0xfffc,
	.boot_params	= 0xa0000100,
	.map_io		= htcuniversal_map_io,
	.init_irq	= htcuniversal_init_irq,
	.init_machine	= htcuniversal_init,
	.timer		= &pxa_timer,
MACHINE_END
