/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * h3600 atmel micro companion support, key subdevice
 * based on previous kernel 2.4 version 
 * Author : Alessandro Gardich <gremlin@gremlin.it>
 *
 */


#include <linux/module.h>
#include <linux/version.h>

#include <linux/init.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/pm.h>
#include <linux/sysctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/input.h>
#include <linux/platform_device.h>

#include <asm/arch/hardware.h>

#include <asm/arch/h3600.h>
#include <asm/arch/SA-1100.h>

#define TX_BUF_SIZE	32
#define RX_BUF_SIZE	16
#define CHAR_SOF	0x02

/*--- keys ---*/
static struct input_dev *micro_key_input;

#define NUM_KEYS 10 
int keycodes[NUM_KEYS] = {
	KEY_RECORD,   // 1:  Record button
	KEY_CALENDAR, // 2:  Calendar
	KEY_CONTACTS, // 3:  Contacts (looks like Outlook)
	KEY_EMAIL,    // 4:  Envelope (Q on older iPAQs)
	KEY_HP,       // 5:  Start (looks like swoopy arrow)
	KEY_UP,       // 6:  Up
	KEY_RIGHT,    // 7:  Right
	KEY_LEFT,     // 8:  Left
	KEY_DOWN      // 9:  Down
};

static void micro_key_receive (int len, unsigned char *data)
{
	int key, down;

	if (0) printk(KERN_ERR "micro key receive\n");

	down = 0x80 & data[0];
	key  = 0x7f & data[0]; 

	if (key < NUM_KEYS ) 
		input_report_key(micro_key_input, keycodes[key], down);
}

typedef struct { 
	char	test[8];
	void    (*handle)( int len, unsigned char *data );
} micro_private_t;

static micro_private_t *p_micro;

static int micro_key_probe (struct device *dev)
{
	int i;
	
	if (0) printk(KERN_ERR "micro key probe : begin\n");

	micro_key_input = input_allocate_device();


	init_input_dev(micro_key_input);

	micro_key_input->evbit[0] = BIT(EV_KEY);
	for (i = 0; i < NUM_KEYS; i++) {
		set_bit (keycodes[i], micro_key_input->keybit);
	}

	micro_key_input->name = "h3600 micro keys";

        input_register_device (micro_key_input);

	if (0) printk(KERN_ERR "micro key probe : end\n");
	/*--- test ---*/
	{
		struct platform_device *plat;
		plat = to_platform_device (dev);

		p_micro = plat->dev.platform_data; 

		sprintf(p_micro->test,"kbd"); 
		p_micro->handle = micro_key_receive;
	}

	return 0;
}

static int micro_key_remove (struct device *dev)
{
        input_unregister_device (micro_key_input);
	{
		struct platform_device *plat;
		plat = to_platform_device (dev);

		p_micro = plat->dev.platform_data; 

		sprintf(p_micro->test,"---"); 
		p_micro->handle = NULL;
	}

        return 0;
}

static int micro_key_suspend ( struct device *dev, u32 state, u32 level) 
{
	return 0;
}

static int micro_key_resume ( struct device *dev, u32 state, u32 level) 
{
	return 0;
}

struct device_driver micro_key_device_driver = {
	.name    = "h3600-micro-keys",
	.bus     = &platform_bus_type,
	.probe   = micro_key_probe,
	.remove  = micro_key_remove,
	.suspend = micro_key_suspend,
	.resume  = micro_key_resume
};

static int micro_key_init (void) 
{
	h3600_micro_dummy_dep();
	/*if (!request_module("micro")) */
	return driver_register(&micro_key_device_driver);
}

static void micro_key_cleanup (void) 
{
	driver_unregister (&micro_key_device_driver);
}

module_init (micro_key_init);
module_exit (micro_key_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("gremlin.it");
MODULE_DESCRIPTION("driver for iPAQ Atmel micro keys");


