/*
 * PXA serial driver private interface
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 */

#ifndef _PXA_SERIAL_H
#define _PXA_SERIAL_H

#define PXA_SERIAL_TX 1
#define PXA_SERIAL_RX 2

#include <linux/tty.h>
#include <linux/serial_core.h>

struct platform_pxa_serial_funcs;

struct uart_pxa_port {
	struct uart_port        port;
	unsigned char           ier;
	unsigned char           lcr;
	unsigned char           mcr;
	unsigned int            lsr_break_flag;
	unsigned int		cken;
	char			*name;
	struct platform_pxa_serial_funcs *pf;
};

/* A pointer to such a structure can be contained in the platform_data
 * field of every PXA UART platform_device. If the field is NULL, the
 * serial port works as usual.
 *
 * For the sake of simplicity/performance no one of the function pointers
 * in the structure below can be NULL.
 */
struct platform_pxa_serial_funcs {
	/* Platform-specific function to initialize whatever is connected
         to this serial port... enable=1 -> enable transceiver,
         0 -> disable transceiver. */
	void (*configure) (struct uart_pxa_port *up, int enable);
        /* Platform-specific function to enable or disable the individual
           transmitter/receiver submodules. On transceivers without echo
           cancellation (e.g. SIR) transmitter always has priority, e.g.
           if both bits are set, only the transmitter is enabled. */
        void (*set_txrx) (struct uart_pxa_port *up, int txrx);
	/* Get the current state of tx/rx (see bitflags above) */
	int (*get_txrx) (struct uart_pxa_port *up);
};

/*
 * The variables below are located in arch/arm/mach-pxa/generic.c
 * Machine-specific code may want to put a pointer to a static
 * platform_pxa_serial_funcs structure in the dev.platform_data
 * field of the respective port device.
 */
extern struct platform_device ffuart_device;
extern struct platform_device btuart_device;
extern struct platform_device stuart_device;
extern struct platform_device hwuart_device;

#endif /* _PXA_SERIAL_H */
