/*
 *  linux/include/leds.h
 *
 *  Copyright (C) 2005 John Lenz <lenz@cs.wisc.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Driver model for leds
 */
#ifndef ASM_ARM_LEDS_H
#define ASM_ARM_LEDS_H

#include <linux/device.h>

struct led_device;

struct led_properties {
	struct module *owner;

	/* Read-only name for this led */
	char *name;
	
	/* Color of the led.  For multiple color leds, the color names should
	 * be seperated by a "/".  For example, "amber/green".
	 * This is read-only.
	 */
	char *color;
	
	/* For multi-colored leds, these function are called to manipulate the
	 * current color. The integer value should be the position in the above
	 * list of colors. For a single color led, set equal to NULL.
	 */
	int (*color_get)(struct device *, struct led_properties *props);
	void (*color_set)(struct device *, struct led_properties *props, int value);

	/* These functions manipulate the brightness of the led.
	 * Values are between 0-100 */
	int (*brightness_get)(struct device *, struct led_properties *props);
	void (*brightness_set)(struct device *, struct led_properties *props, int value);

	/* These functions manipulate the blink frequency for this led.  Values
	 * are in milliseconds.  If these functions are set to NULL, the ledscode
	 * will blink this led using a kernel timer. Setting a value of zero disables
	 * the blinking. */
	unsigned long (*blink_frequency_get)(struct device *, struct led_properties *props);
	void (*blink_frequency_set)(struct device *, struct led_properties *props, unsigned long value);
	
	/* private structure */
	struct led_device *led_dev;
};

int leds_device_register(struct device *dev, struct led_properties *props);
void leds_device_unregister(struct led_properties *props);

int leds_acquire(struct led_properties *led);
void leds_release(struct led_properties *led);
int leds_set_frequency(struct led_properties *led, unsigned long frequency);

struct led_interface {
	int (*add)(struct device *dev, struct led_properties *led);
	void (*remove)(struct device *dev, struct led_properties *led);

	struct list_head node;
};
int leds_interface_register(struct led_interface *interface);
void leds_interface_unregister(struct led_interface *interface);

#endif
