
/*
 * PXA i2Sound code for HTC Universal
 * It uses PXA2xx i2Sound and AK4641 modules
 *
 * Copyright (c) 2005 Todd Blumer, SDG Systems, LLC
 *
 * Based on code:
 *   Copyright (c) 2005 Giorgio Padrin giorgio@mandarinlogiq.org
 *   Copyright (c) 2004,2005 Matthew Reimer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * History:
 * 
 * 2005-06	Todd Blumer		Initial release
 */

#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/hardware.h>
#include <linux/pm.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <asm/arch/pxa-regs.h>
#include <linux/soc/asic3_base.h>
#include <asm/arch/htcuniversal-gpio.h>
#include <asm/arch/htcuniversal-asic.h>

#include "pxa2xx-i2sound.h"
#include <sound/ak4641.h>

static int earphone_out = 0;
static int output_enabled;

static void
snd_htcuniversal_audio_set_codec_power( int mode )
{
//	SET_HX4700_GPIO( CODEC_ON, mode );
	 ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_AUDIO_PWR_ON , mode?(1<<GPIOA_AUDIO_PWR_ON):0);
}

static void
snd_htcuniversal_audio_set_codec_reset( int mode )
{
	 printk( KERN_NOTICE "snd_htcuniversal_audio_set_codec_reset: %d\n", mode );
//	SET_HX4700_GPIO_N( CODEC_PDN, mode );
}

static void
snd_htcuniversal_audio_set_mic_power( int mode )
{
	 printk( KERN_NOTICE "snd_htcuniversal_audio_set_mic_power: %d\n", mode );
	snd_ak4641_mic( mode );
	ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_SPKMIC_PWR1_ON , mode?(1<<GPIOA_SPKMIC_PWR1_ON):0);
	ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_SPKMIC_PWR2_ON , mode?(1<<GPIOA_SPKMIC_PWR2_ON):0);
}

static void
snd_htcuniversal_audio_set_speaker_power( int mode )
{
	 printk( KERN_NOTICE "snd_htcuniversal_audio_set_speaker_power: %d\n", mode );
	/* speaker shutdown (0=shutdown)*/
//	SET_HX4700_GPIO( SPK_SD_N, mode );
	ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_SPKMIC_PWR1_ON , mode?(1<<GPIOA_SPKMIC_PWR1_ON):0);
	ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_SPKMIC_PWR2_ON , mode?(1<<GPIOA_SPKMIC_PWR2_ON):0);
}

static void
snd_htcuniversal_audio_set_output( void )
{
	if (output_enabled) {
//		earphone_out = GET_HX4700_GPIO( EARPHONE_DET_N );
		earphone_out = (((ipaq_asic3_read_gpio_status_b( &htcuniversal_asic3.dev )) & (1<<GPIOB_EARPHONE_N)) == 0);
		/*
		 * set both headphones and speaker since this may
		 * be state change (e.g. headphone insertion)
		 */
//		SET_HX4700_GPIO( HP_DRIVER, earphone_out == 0 );
		ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_EARPHONE_PWR_ON , earphone_out?(1<<GPIOA_EARPHONE_PWR_ON):0);
		snd_htcuniversal_audio_set_speaker_power( earphone_out != 0 );
	}
}

static int
earphone_isr(int irq, void *dev_id, struct pt_regs *regs)
{
	snd_htcuniversal_audio_set_output();
	return IRQ_HANDLED;
}

static void
snd_htcuniversal_audio_earphone_detection_on(void)
{
	int irq = asic3_irq_base( &htcuniversal_asic3.dev ) + ASIC3_GPIOB_IRQ_BASE + GPIOB_EARPHONE_N;
        if (request_irq( irq, earphone_isr, SA_INTERRUPT,
			    "htcuniversal Headphone Jack", NULL ) != 0)
		return;
	set_irq_type( irq, IRQT_FALLING );
}

static void
snd_htcuniversal_audio_earphone_detection_off(void)
{
	int irq = asic3_irq_base( &htcuniversal_asic3.dev ) + ASIC3_GPIOB_IRQ_BASE + GPIOB_EARPHONE_N;
        free_irq( irq, NULL );
}


static int
snd_htcuniversal_audio_activate(void)
{
	snd_pxa2xx_i2sound_i2slink_get(); /* I2SLINK clocks required always on
					   * by AK4641 module */ 
	
	snd_htcuniversal_audio_set_codec_reset(1);
	snd_htcuniversal_audio_set_codec_power(1);
	mdelay(1);
	snd_htcuniversal_audio_set_codec_reset(0);
	
	request_module("i2c-pxa");
	
	if (snd_ak4641_activate() == 0) { 
		snd_htcuniversal_audio_earphone_detection_on();
#if 0
		earphone_out = GET_HX4700_GPIO( EARPHONE_DET_N );
		snd_htcuniversal_audio_set_speaker_power( earphone_out != 0 );
		SET_HX4700_GPIO( HP_DRIVER, earphone_out == 0 );
#else
		earphone_out = (((ipaq_asic3_read_gpio_status_b( &htcuniversal_asic3.dev )) & (1<<GPIOB_EARPHONE_N)) == 0);
		ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_EARPHONE_PWR_ON , earphone_out?(1<<GPIOA_EARPHONE_PWR_ON):0);
		snd_htcuniversal_audio_set_speaker_power( earphone_out != 0 );
#endif
		return 0;
	}
	else {
		snd_htcuniversal_audio_set_codec_power(0);
		snd_pxa2xx_i2sound_i2slink_free();
		return 1;
	}
}

static void
snd_htcuniversal_audio_deactivate(void)
{
	snd_htcuniversal_audio_earphone_detection_off();
	
	snd_htcuniversal_audio_set_speaker_power(0);
	snd_htcuniversal_audio_set_mic_power(0);
	snd_htcuniversal_audio_set_codec_power(0);
	
	snd_ak4641_deactivate();
	
	snd_pxa2xx_i2sound_i2slink_free();
}

static int
snd_htcuniversal_audio_open_stream(int stream)
{
	// printk( KERN_NOTICE "snd_htcuniversal_audio_open_stream: %x\n", stream );
	if (stream == SNDRV_PCM_STREAM_PLAYBACK) {
		output_enabled = 1;
		snd_htcuniversal_audio_set_output();
	}
	else
		snd_htcuniversal_audio_set_mic_power(1);
	return 0;
}

static void
snd_htcuniversal_audio_close_stream(int stream)
{
	// printk( KERN_NOTICE "snd_htcuniversal_audio_close_stream: %x\n", stream );
	if (stream == SNDRV_PCM_STREAM_PLAYBACK) {
		output_enabled = 0;
		snd_htcuniversal_audio_set_output();
	}
	else
		snd_htcuniversal_audio_set_mic_power(0);
}				

#ifdef CONFIG_PM

static int
snd_htcuniversal_audio_suspend(pm_message_t state)
{
	snd_ak4641_no_plop(1);
	snd_htcuniversal_audio_set_speaker_power(0);
	ak4641_command(ak4641_i2c_get_client(), I2C_AK4641_CLOSE, NULL);	
//	SET_HX4700_GPIO( HP_DRIVER, 0 );
	ipaq_asic3_set_gpio_out_a(&htcuniversal_asic3.dev, 1<<GPIOA_EARPHONE_PWR_ON , 0);
	snd_htcuniversal_audio_set_codec_power(0);
	mdelay(1);
	return 0;
}

static int
snd_htcuniversal_audio_resume(void)
{
	// printk( KERN_NOTICE "snd_pxa2xx_i2sound_htcuniversal_resume\n" );
	snd_htcuniversal_audio_set_codec_reset(1);
	snd_htcuniversal_audio_set_codec_power(1);
	mdelay(1);
	snd_htcuniversal_audio_set_codec_reset(0);
	snd_htcuniversal_audio_set_output();
	ak4641_command(ak4641_i2c_get_client(), I2C_AK4641_OPEN, NULL);
	snd_ak4641_no_plop(0);
	return 0;
}

#endif

static struct snd_pxa2xx_i2sound_board htcuniversal_audio = {
	.name			= "HTC Universal Audio",
	.desc			= "HTC Universal Audio [codec AK4641]",
	.acard_id		= "htcuniversal-audio",
	.info			= SND_PXA2xx_I2SOUND_INFO_CLOCK_FROM_PXA |
				  SND_PXA2xx_I2SOUND_INFO_CAN_CAPTURE,
	.activate		= snd_htcuniversal_audio_activate,
	.deactivate		= snd_htcuniversal_audio_deactivate,
	.open_stream		= snd_htcuniversal_audio_open_stream,
	.close_stream		= snd_htcuniversal_audio_close_stream,
	.add_mixer_controls	= snd_ak4641_mixer_create,
#ifdef CONFIG_PM	
	.suspend		= snd_htcuniversal_audio_suspend,
	.resume			= snd_htcuniversal_audio_resume
#endif
};


static int __init
snd_pxa2xx_i2sound_htcuniversal_init(void)
{
	printk( KERN_NOTICE "HTC Universal i2Sound Module\n" );
	return snd_pxa2xx_i2sound_card_activate( &htcuniversal_audio );
}

static void __exit
snd_pxa2xx_i2sound_htcuniversal_free(void)
{
	snd_pxa2xx_i2sound_card_deactivate();
}

module_init(snd_pxa2xx_i2sound_htcuniversal_init);
module_exit(snd_pxa2xx_i2sound_htcuniversal_free);

MODULE_AUTHOR("Todd Blumer, SDG Systems, LLC");
MODULE_DESCRIPTION("Audio Support for HTC Universal");
MODULE_LICENSE("GPL");

/* vim600: set noexpandtab sw=8 ts=8 :*/

